package org.artifactory.security;

import org.artifactory.addon.build.BuildAddon;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.common.ConstantValues;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.config.CentralConfigDescriptorImpl;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.spring.ArtifactoryApplicationContext;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import static org.artifactory.repo.config.RepoConfigDefaultValues.EXAMPLE_REPO_KEY;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

/**
 * @author Mor Merhav
 */
public class FirstStartupHelperTest extends ArtifactoryHomeBoundTest {

    @Mock
    private ArtifactoryApplicationContext context;
    @Mock
    private RepositoryService repositoryService;

    @BeforeClass
    public void beforeClass() {
        MockitoAnnotations.initMocks(this);
        ArtifactoryContextThreadBinder.bind(context);
        when(context.getRepositoryService()).thenReturn(repositoryService);
    }

    @AfterClass
    public void afterClass() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testOnlyDefaultReposExistFilterPipelineRepoReturnTrue() {
        CentralConfigDescriptor centralConfigDescriptor = new CentralConfigDescriptorImpl();
        LocalRepoDescriptor exampleRepo = new LocalRepoDescriptor();
        exampleRepo.setKey(EXAMPLE_REPO_KEY);
        LocalRepoDescriptor buildInfoRepo = new LocalRepoDescriptor();
        buildInfoRepo.setKey(BuildAddon.BUILD_INFO_REPO_NAME);
        LocalRepoDescriptor pipelinesRepo = new LocalRepoDescriptor();
        buildInfoRepo.setKey(ConstantValues.defaultPipelinesRepoKey.getString());

        centralConfigDescriptor.getLocalRepositoriesMap().put(EXAMPLE_REPO_KEY, exampleRepo);
        centralConfigDescriptor.getLocalRepositoriesMap().put(BuildAddon.BUILD_INFO_REPO_NAME, buildInfoRepo);
        centralConfigDescriptor.getLocalRepositoriesMap().put(ConstantValues.defaultPipelinesRepoKey.getString(), pipelinesRepo);

        when(repositoryService.getArtifactCount(any())).thenReturn(0L);

        boolean onlyDefaultReposExist = FirstStartupHelper.onlyDefaultReposExist(centralConfigDescriptor);
        assertTrue(onlyDefaultReposExist);
    }

    @Test
    public void testNotOnlyDefaultReposExistReturnFalse() {
        CentralConfigDescriptor centralConfigDescriptor = new CentralConfigDescriptorImpl();
        LocalRepoDescriptor exampleRepo = new LocalRepoDescriptor();
        exampleRepo.setKey(EXAMPLE_REPO_KEY);
        LocalRepoDescriptor buildInfoRepo = new LocalRepoDescriptor();
        buildInfoRepo.setKey(BuildAddon.BUILD_INFO_REPO_NAME);
        LocalRepoDescriptor nonDefaultRepo = new LocalRepoDescriptor();
        nonDefaultRepo.setKey("nonDefaultRepo");

        centralConfigDescriptor.getLocalRepositoriesMap().put(EXAMPLE_REPO_KEY, exampleRepo);
        centralConfigDescriptor.getLocalRepositoriesMap().put(BuildAddon.BUILD_INFO_REPO_NAME, buildInfoRepo);
        centralConfigDescriptor.getLocalRepositoriesMap().put("nonDefaultRepo", nonDefaultRepo);

        when(repositoryService.getArtifactCount(any())).thenReturn(0L);

        boolean onlyDefaultReposExist = FirstStartupHelper.onlyDefaultReposExist(centralConfigDescriptor);
        assertFalse(onlyDefaultReposExist);
    }

    @Test
    public void testDefaultReposNotEmptyReturnFalse() {
        CentralConfigDescriptor centralConfigDescriptor = new CentralConfigDescriptorImpl();
        LocalRepoDescriptor exampleRepo = new LocalRepoDescriptor();
        exampleRepo.setKey(EXAMPLE_REPO_KEY);
        LocalRepoDescriptor buildInfoRepo = new LocalRepoDescriptor();
        buildInfoRepo.setKey(BuildAddon.BUILD_INFO_REPO_NAME);

        centralConfigDescriptor.getLocalRepositoriesMap().put(EXAMPLE_REPO_KEY, exampleRepo);
        centralConfigDescriptor.getLocalRepositoriesMap().put(BuildAddon.BUILD_INFO_REPO_NAME, buildInfoRepo);

        when(repositoryService.getArtifactCount(any())).thenReturn(1L);

        boolean onlyDefaultReposExist = FirstStartupHelper.onlyDefaultReposExist(centralConfigDescriptor);
        assertFalse(onlyDefaultReposExist);
    }
}