package org.artifactory.security.ldap;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.CoreAddons;
import org.artifactory.addon.LdapGroupAddon;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.security.ldap.LdapUserAttributes;
import org.artifactory.descriptor.security.ldap.LdapSetting;
import org.artifactory.descriptor.security.ldap.SearchPattern;
import org.artifactory.spring.InternalArtifactoryContext;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mockito;
import org.springframework.ldap.core.DirContextAdapter;
import org.springframework.ldap.core.support.LdapContextSource;
import org.springframework.security.ldap.search.LdapUserSearch;
import org.springframework.util.ReflectionUtils;
import org.testng.Assert;
import org.testng.annotations.Test;

import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttributes;
import java.lang.reflect.Field;
import java.util.List;

/**
 * @author BarakH
 */
public class LdapServiceImplTest extends ArtifactoryHomeBoundTest {

    @Test
    public void testGetUserWithAttributesFromDomainWithCustomFilerPositive() throws NamingException {
        LdapServiceImpl ldapService = new LdapServiceImpl();
        LdapSetting ldapSetting = Mockito.mock(LdapSetting.class);
        SearchPattern searchPattern = new SearchPattern();
        Mockito.when(ldapSetting.getSearch()).thenReturn(searchPattern);
        Mockito.when(ldapSetting.isEnabled()).thenReturn(true);
        Mockito.when(ldapSetting.getLdapUrl()).thenReturn("ldap://never.mind/cn=dont,cd=care");
        AddonsManager addonsManager = Mockito.mock(AddonsManager.class);
        CoreAddons coreAddons = Mockito.mock(CoreAddons.class);
        Mockito.when(coreAddons.isAol()).thenReturn(false);
        Mockito.when(addonsManager.addonByType(CoreAddons.class)).thenReturn(coreAddons);
        Field field = ReflectionUtils.findField(LdapServiceImpl.class, "addonsManager");
        ReflectionUtils.makeAccessible(field);
        ReflectionUtils.setField(field, ldapService, addonsManager);
        InternalArtifactoryContext context = Mockito.mock(InternalArtifactoryContext.class);
        LdapGroupAddon ldapGroupAddon = Mockito.mock(LdapGroupAddon.class);
        LdapUserSearch ldapUserSearch = Mockito.mock(LdapUserSearch.class);
        Mockito.when(ldapGroupAddon
                .getLdapUserSearches(Mockito.any(LdapContextSource.class), Mockito.eq(ldapSetting), Mockito.eq(false)))
                .thenReturn(List.of(ldapUserSearch));
        Mockito.when(addonsManager.addonByType(LdapGroupAddon.class)).thenReturn(ldapGroupAddon);
        Mockito.when(context.beanForType(AddonsManager.class)).thenReturn(addonsManager);
        ArtifactoryContextThreadBinder.bind(context);
        DirContextAdapter dirContextAdapter = Mockito.mock(DirContextAdapter.class);
        Mockito.when(ldapUserSearch.searchForUser("yo")).thenReturn(dirContextAdapter);
        Mockito.when(dirContextAdapter.getNameInNamespace()).thenReturn("yoyo");
        Attributes attributes = new BasicAttributes();
        attributes.put("attr", "val");
        Mockito.when(dirContextAdapter.getAttributes()).thenReturn(attributes);

        //Verify the user got back with the attribute
        LdapUserAttributes userAttributesResult = ldapService.getUserWithAttributesFromDomainWithCustomFiler(ldapSetting, "yo");
        Assert.assertEquals("val", userAttributesResult.getAttributes().get("attr").get(0).toString());

        //Verify we are getting the user from the cache
        LdapUserAttributes userAttributesSecondResult = ldapService.getUserWithAttributesFromDomainWithCustomFiler(ldapSetting, "yo");
        attributes = new BasicAttributes();
        attributes.put("attr2", "val2");
        Mockito.when(dirContextAdapter.getAttributes()).thenReturn(attributes);
        Assert.assertEquals("val", userAttributesSecondResult.getAttributes().get("attr").get(0).toString());
        Assert.assertNull(userAttributesSecondResult.getAttributes().get("attr2"));
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testGetUserWithAttributesFromDomainWithCustomFilerNegative(){
        LdapServiceImpl ldapService = new LdapServiceImpl();
        LdapSetting ldapSetting = Mockito.mock(LdapSetting.class);
        SearchPattern searchPattern = new SearchPattern();
        Mockito.when(ldapSetting.getSearch()).thenReturn(searchPattern);
        Assert.assertNull(ldapService.getUserWithAttributesFromDomainWithCustomFiler(null, "filter"));
    }
}