package org.artifactory.security.ldap;

import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.security.ldap.LdapService;
import org.artifactory.api.security.ldap.LdapUser;
import org.artifactory.api.security.ldap.LdapUserAttributes;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.descriptor.security.SecurityDescriptor;
import org.artifactory.descriptor.security.ldap.LdapSetting;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.junit.Assert;
import org.mockito.Mockito;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttributes;
import java.util.Collections;

/**
 * @author BarakH
 */
public class LdapUtilsTest extends ArtifactoryHomeBoundTest {

    @Test(dataProvider = "provideAttributes")
    public void testFindSettingsForActiveUser(String attrID, String attrVal, boolean expectedNull) {
        String userName = "user";
        CentralConfigService centralConfigService = Mockito.mock(CentralConfigService.class);
        MutableCentralConfigDescriptor descriptor = Mockito.mock(MutableCentralConfigDescriptor.class);
        SecurityDescriptor securityDescriptor = Mockito.mock(SecurityDescriptor.class);
        LdapSetting ldapSetting = Mockito.mock(LdapSetting.class);
        Mockito.when(securityDescriptor.getLdapSettings()).thenReturn(Collections.singletonList(ldapSetting));
        Mockito.when(descriptor.getSecurity()).thenReturn(securityDescriptor);
        Mockito.when(centralConfigService.getMutableDescriptor()).thenReturn(descriptor);
        LdapService ldapService = Mockito.mock(LdapService.class);
        LdapUser ldapUser = new LdapUser(userName, userName);
        Attributes attributes = new BasicAttributes();
        attributes.put(attrID, attrVal);
        attributes.put("lockoutDuration", "-600000000");
        LdapUserAttributes ldapUserAttributes = new LdapUserAttributes(ldapUser, attributes);
        Mockito.when(ldapService.getUserWithAttributesFromDomain(ldapSetting, userName)).thenReturn(ldapUserAttributes);
        Mockito.when(ldapService.getUserWithAttributesFromDomainWithCustomFiler(ldapSetting, "objectClass=domain"))
                .thenReturn(ldapUserAttributes);
        Mockito.when(ldapSetting.isEnabled()).thenReturn(true);

        LdapSetting result = LdapUtils.findSettingsForActiveUser(userName, centralConfigService, ldapService);
        if (expectedNull) {
            Assert.assertNull(result);
        } else {
            Assert.assertNotNull(result);
        }
    }

    @DataProvider
    private static Object[][] provideAttributes() {
        return new Object[][]{
                {"lockoutTime", "0", false},
                {"lockoutTime", "132282298867991751", false},
                {"lockoutTime", "999999999999999999", true},
                {"notAnAttribute", "val", false},
                {"pwdAccountLockedTime", "1234", true},
                {"userAccountControl", "66082", true},
                {"userAccountControl", "66050", true},
                {"userAccountControl", "514", true},
                {"userAccountControl", "546", true},
                {"userAccountControl", "262658", true},
                {"userAccountControl", "262690", true},
                {"userAccountControl", "328194", true},
                {"userAccountControl", "328226", true},
                {"userAccountControl", "544", false},
                {"userAccountControl", "66048", false},
                {"userAccountControl", "262656", false},
        };
    }
}