:: This script starts Artifactory and dependent services
:: 

@echo off
setlocal EnableDelayedExpansion
title artifactory

goto initialize

:setVariables
    CALL %~dp0\artifactoryEnv.bat

    set METADATA_SERVICE_NAME=metadata
    set REPLICATOR_SERVICE_NAME=replicator
    set ROUTER_SERVICE_NAME=router
    set FRONTEND_SERVICE_NAME=frontend
EXIT /B 0

:initialize 
    :: Sets all variables required for starting Artifactory and dependent services
    CALL :setVariables
    
    :: Interpret command mode from arguments and call appropriate action
    if "%~1" equ "start" (
        shift /1
        goto detachProcess
        )
    if "%~1" equ "stop" (
        shift /1
        goto stopAll
        goto endexec
        )
    if "%~1" equ "spawn" (
        shift /1
        goto spawnAndStart
        goto endexec
        )
    if "%~1" equ "startAll" (
        CALL %~dp0\artifactoryCommon.bat :log "JF_PRODUCT_HOME is - "%JF_PRODUCT_HOME%
        shift /1
        goto startAll
        goto endexec
        )
    goto detachProcess
EXIT /B 0

:detachProcess
    CALL <NUL "%~f0" spawn %*
EXIT /B 0

:spawnAndStart
    cmd /d /c "%~f0" startAll %*
    goto gracefulShutDown
    goto endexec
EXIT /B 0 

:: Actual start/stop functions 
:startAll
    SET scriptName=%~n0%~x0
    CALL %~dp0\artifactoryCommon.bat :log "Starting all services"
    CALL :startupActions || goto :error
    SET scriptName=%~n0%~x0
    CALL :startRouter
    CALL :startMetadata
    CALL :startReplicator
    CALL :startFrontend
    CALL :startArtifactory
EXIT /B 0

:stopAll
    :: Stop dependent services
    SET scriptName=%~n0%~x0
    CALL :stopRouter
    CALL :stopMetadata
    CALL :stopReplicator
    CALL :stopFrontend
    CALL :stopArtifactory
EXIT /B 0

:gracefulShutDown
    SET scriptName=%~n0%~x0
    CALL %~dp0\artifactoryCommon.bat :log "Starting graceful shutdown"
    CALL :stopAll
EXIT /B 0

:: Individual start/stop calls 
:startRouter 
    CALL %~dp0\artifactoryCommon.bat :log "Starting router"
    CALL %JF_PRODUCT_HOME%\app\%ROUTER_SERVICE_NAME%\bin\%ROUTER_SERVICE_NAME% start
EXIT /B 0
:stopRouter 
    CALL %JF_PRODUCT_HOME%\app\%ROUTER_SERVICE_NAME%\bin\%ROUTER_SERVICE_NAME% stop
EXIT /B 0

:startReplicator
    set RUN_REPLICATOR=false
    CALL %~dp0\artifactoryCommon.bat :runReplicator
    if "%RUN_REPLICATOR%" == "true" (
        CALL %~dp0\artifactoryCommon.bat :log "Starting replicator"
        :: TODO - check errorlevel
        CALL %JF_PRODUCT_HOME%\app\%REPLICATOR_SERVICE_NAME%\bin\%REPLICATOR_SERVICE_NAME% start
    ) else (
        CALL %~dp0\artifactoryCommon.bat :log "Replicator is disabled, %REPLICATOR_SERVICE_NAME% is not started"
    )
EXIT /B 0
:stopReplicator 
    set RUN_REPLICATOR=false
    CALL %~dp0\artifactoryCommon.bat :runReplicator
    if "%RUN_REPLICATOR%" == "true" (
        CALL %JF_PRODUCT_HOME%\app\%REPLICATOR_SERVICE_NAME%\bin\%REPLICATOR_SERVICE_NAME% stop
    )
EXIT /B 0

:startFrontend 
    CALL %~dp0\artifactoryCommon.bat :log "Starting frontend"
    CALL %JF_PRODUCT_HOME%\app\%FRONTEND_SERVICE_NAME%\bin\%FRONTEND_SERVICE_NAME% start
EXIT /B 0
:stopFrontend 
    CALL %JF_PRODUCT_HOME%\app\%FRONTEND_SERVICE_NAME%\bin\%FRONTEND_SERVICE_NAME% stop
EXIT /B 0

:startArtifactory 
    CALL %CATALINA_HOME%\bin\catalina.bat stop >nul 2>&1
    CALL %~dp0\artifactoryCommon.bat :log "Starting artifactory"
    CALL %CATALINA_HOME%\bin\catalina.bat run
EXIT /B 0
:stopArtifactory 
    CALL %~dp0\artifactoryCommon.bat :log "Stopping artifactory"
    CALL %CATALINA_HOME%\bin\catalina.bat stop 2>nul
EXIT /B 0

:startMetadata 
    CALL %~dp0\artifactoryCommon.bat :log "Starting metadata"
    CALL %JF_PRODUCT_HOME%\app\%METADATA_SERVICE_NAME%\bin\%METADATA_SERVICE_NAME% start
EXIT /B 0
:stopMetadata 
    CALL %JF_PRODUCT_HOME%\app\%METADATA_SERVICE_NAME%\bin\%METADATA_SERVICE_NAME% stop
EXIT /B 0

:startupActions
    SET scriptName=artifactoryCommon.bat
    CALL %~dp0\artifactoryCommon.bat :checkJavaVersion || goto error
    CALL %~dp0\artifactoryCommon.bat :prepareTomcat
    CALL %~dp0\artifactoryCommon.bat :configureServerXml || goto error
    CALL %~dp0\artifactoryCommon.bat :syncEtc
    CALL %~dp0\artifactoryCommon.bat :addCertsToJavaKeystore
    CALL %~dp0\artifactoryCommon.bat :setNodeDetails
    CALL %~dp0\artifactoryCommon.bat :setRouterTopology
EXIT /B 0

:error
    @endlocal
exit /b 1

:endexec
    @endlocal
exit /b 0
