@echo off
:: Add generic methods which might clutter or which can be reused accross multiple scripts.
:: Things to note before adding a method in this file,
::  - It will get complicated if recursion logic is used with sub routines

:: Assign method name and its parameters
set method=%1
set ALL_BUT_FIRST=

for /f "tokens=1,* delims= " %%a in ("%*") do set ALL_BUT_FIRST=%%b
:: Use /1 to retain %0 values intact
shift /1

goto %method% %ALL_BUT_FIRST%
goto :eof

:: Set extra java options from system.yaml
:setExtraJavaOpts
    set EXTRA_JAVA_OPTIONS=
    CALL %~dp0\systemYamlHelper.bat :getSystemValue shared.extraJavaOpts
    set EXTRA_JAVA_OPTIONS=!YAML_VALUE!
exit /b 0

:initNode
    set NODE="%JF_PRODUCT_HOME%\app\third-party\node\node.exe"
    set NODE_ENV=PRODUCTION #TODO (matank): should be removed after JFUI-1372 will be resolved
exit /b 0

:setRouterTopology
    set "JF_ROUTER_TOPOLOGY_LOCAL_REQUIREDSERVICETYPES=jfrt,jfac,jfmd,jffe"
    set RUN_REPLICATOR=false
    CALL :runReplicator
    if "%RUN_REPLICATOR%" == "true" (
        set "JF_ROUTER_TOPOLOGY_LOCAL_REQUIREDSERVICETYPES=!JF_ROUTER_TOPOLOGY_LOCAL_REQUIREDSERVICETYPES!,jfxfer"
    )
exit /b 0

:runReplicator
    CALL %~dp0\systemYamlHelper.bat :getSystemValue replicator.enabled false
    set replicatorEnabled=!YAML_VALUE!
    if exist %JF_PRODUCT_HOME%\app\%REPLICATOR_SERVICE_NAME%\bin\%REPLICATOR_SERVICE_NAME%.bat if "!replicatorEnabled!" == "true" (
        set RUN_REPLICATOR=true
    )
EXIT /B 0

:: Enable user to change java home through system.yaml
:setJavaHome
    CALL %~dp0\systemYamlHelper.bat :getSystemValue shared.javaHome "%JF_PRODUCT_HOME%\app\third-party\java"
    set JAVA_HOME=!YAML_VALUE!
    set "PATH=!PATH!;!JAVA_HOME!\bin"
exit /b 0

:checkJavaVersion
    setlocal EnableDelayedExpansion
    set supportedVersion=11
    for /f "tokens=3" %%g in ('%JAVA_HOME%\bin\java.exe -version 2^>^&1 ^| findstr /i "version"') do (
        set CurVer=%%g
    )

    :javaAvailable
    set t=%CurVer:"=%
    for /f "delims=. tokens=1-3" %%v in ("%t%") do (
        set major=%%v
        set minor=%%w
    )

    if "%major%" == "%supportedVersion%" goto :javaOk
    
    set JF_ERROR_MSG="Java version must be %supportedVersion%, you have %CurVer% configured"
    goto errorExit
    
    :javaOk
    @endlocal
exit /b 0

:setNodeDetails
    set sharedNodeId=
    set sharedNodeName=
    set sharedNodeIp=

    set ip_address_string="IPv4 Address"
    :: Uncomment the following line when using older versions of Windows without IPv6 support (by removing "rem")
    rem set ip_address_string="IP Address"
    for /f "usebackq tokens=2 delims=:" %%f in (`ipconfig ^| findstr /c:%ip_address_string%`) do (
        set sharedNodeIp=%%f
        rem Remove leading and trailing spaces from the variable
        set sharedNodeIp=!sharedNodeIp: =!
    )

    for /f "usebackq tokens=1 delims= " %%f in (`hostname`) do (
        set sharedNodeName=%%f
        rem Remove leading and trailing spaces from the variable
        set sharedNodeName=!sharedNodeName: =!
        set sharedNodeId=!sharedNodeName!
    )

    call :setNodeProperty shared.node.id !sharedNodeId!
    call :setNodeProperty shared.node.ip !sharedNodeIp!
    call :setNodeProperty shared.node.name !sharedNodeName!
exit /b 0

:setNodeProperty
    set nodeKey=%~1
    set defaultValue=%~2
    set nodeValue=
    set transformedKey=%nodeKey%
    set envValue=

    if not defined nodeKey goto :setNodePropertyEnd

    call %~dp0\systemYamlHelper.bat :transformKeyToEnv "!nodeKey!" transformedKey
    set envValue=!%transformedKey%!

    CALL %~dp0\systemYamlHelper.bat :getSystemValue !nodeKey! NOT_SET false
    set nodeValue=!YAML_VALUE!

    :: use case : not set as envinronment not system.yaml
    if "!nodeValue!" == "NOT_SET" (
        set nodeValue=%defaultValue%
        call :log "Setting %transformedKey% to !nodeValue!"
        set %transformedKey%=!nodeValue!
        goto :setNodePropertyEnd
    )  
    
    :: use case : set in system.yaml and not set as environment variable 
    if not defined !envValue! if not "!nodeValue!" == "NOT_SET" (
        call :log "Found explicit !nodeKey! in system.yaml with value : !nodeValue!"
        set %transformedKey%=!nodeValue!
        goto :setNodePropertyEnd
    )
    
    :: use case : set as environment variable
    if defined !envValue! (
        call :log "Found %transformedKey% set as environment variable with value : !%envValue%!"
    )
    :setNodePropertyEnd
exit /b 0

:addCertsToJavaKeystore
    setlocal EnableDelayedExpansion
    set certsDir=%JF_PRODUCT_HOME%\var\etc\security\keys\trusted
    set packagedJavaHome=%JF_PRODUCT_HOME%\app\third-party\java
    set defaultKeyStorePass=changeit

    :: Return if certs directory does not exist
    if not exist %certsDir% goto :addCertsToJavaKeystoreEnd

    IF NOT DEFINED JAVA_HOME SET "JAVA_HOME=%packagedJavaHome%"

    :: Return if a custom java home is used
    if NOT "%packagedJavaHome%" == "%JAVA_HOME%" (
        call :warn "Found custom JAVA_HOME (%JAVA_HOME%), skipping import of certificates. They are stored in %certsDir%, this needs to be imported manually to your Java cacerts"
        goto :addCertsToJavaKeystoreEnd
    )

    if defined JF_SHARED_SECURITY_KEYSTOREPASSWORD (
        set storePass=%JF_SHARED_SECURITY_KEYSTOREPASSWORD%
    ) else (
        set storePass=%defaultKeyStorePass%
    )

    set keyTool=%JAVA_HOME%\bin\keytool.exe

    if not exist "%keyTool%" (
        call :warn "%keyTool% not found , certificates in %certsDir% directory have to be manually imported to Java cacerts"
        goto :addCertsToJavaKeystoreEnd
    )

    for %%i in (%certsDir%\*) do (
        set certName=%%~ni
        set cert=%%i

        :: Check if alias already installed
        %keyTool% -list -cacerts -storepass %storePass% -alias !certName! >nul 2>&1 
        if !ERRORLEVEL! == 0 (
            call :log "Certificate !cert! already in Java cacerts"
        ) else (
            call :log "Adding !cert! to Java cacerts"
            %keyTool% -importcert -trustcacerts -noprompt -cacerts -storepass %storePass% -file !cert! -alias !certName!
            if NOT ERRORLEVEL == 0 call :warn "Adding !cert! failed!"
        )
    )
    :addCertsToJavaKeystoreEnd
    endlocal
EXIT /B 0

Rem Remove duplicate java options
:removeDuplicateJavaOptions
    setlocal EnableDelayedExpansion
    set sedExec=%JF_PRODUCT_HOME%\app\third-party\sed\bin\sed.exe
    set default_java_options=%1
    set default_java_options=!default_java_options:"=!
    set extra_java_options=%2
    for /F "tokens=*" %%p in ('echo !extra_java_options! ^| !sedExec! "s/=/_/g"') do ( set extra_java_options=%%~p )
    set extra_java_options=!extra_java_options:"=!
    for %%i in (!extra_java_options!) do (
        set k=
        set t=%%i
        Rem Check for Memory parameters
        echo !t! | findstr ".X[mxs][xs].\w*" > nul && set k=!t:~0,4!
        if [!k!] == [] (
            Rem Check for System parameters
            echo !t! | findstr ".D.\w*" > nul && for /F "tokens=1 delims=_" %%j in ('echo !t!') do ( set k=%%~j )
            call :trim k !k!
        )
        if not [!k!] == [] (
            Rem Remove the matching element
            for /F "tokens=*" %%r in ('echo !default_java_options! ^| !sedExec! "s/!k!.\w*//g"') do ( set default_java_options=%%~r )
            call :trim default_java_options !default_java_options!
        )
    )
    endlocal & set UNIQUE_DEFAULT_JAVA_OPTIONS=%default_java_options%
exit /B 0

Rem remove trailing spaces in variables
:trim
    setlocal EnableDelayedExpansion
    set Params=%*
    for /F "tokens=1*" %%a in ("!Params!") do endlocal & set %1=%%b
exit /B 0

:copyFile
    setlocal
    set src=%~1
    set target=%~2
    set overwrite=%~3
    set displayMsg=%~4
 
    set target=%target:"=%
    set src=%src:"=%

    if not exist %src% goto :copyFileEnd
    if not defined displayMsg set "displayMsg=true"
    :: Find and create parent directory of target
    for %%a in ("%target%") do set "parentDir=%%~dpa"
    
    if not exist %parentDir% mkdir %parentDir%
    if exist %target% if NOT %overwrite% == overwrite goto :copyFileEnd

    if "!displayMsg!" == "true" CALL :log "Copying from %src% to %target%"
    xcopy /s /Y %src% %target%* >nul
    
    :copyFileEnd
    @endlocal
EXIT /B 0

:syncEtc
    setlocal
    set srcEtc=%JF_PRODUCT_HOME%\app\misc\etc
    set targetEtc=%JF_PRODUCT_HOME%\var\etc

    :: Use this if there are any configuration files need to be copied over to specific service directory
    set relativeFolders=artifactory

    :: Add comma seperated file list to copy only if not available
    set relativeFilesNoOverwrite=

    :: Add comma seperated file list to overwrite on copy
    set relativeFilesReplace=system.basic-template.yaml,system.full-template.yaml
    
    FOR %%a in ("%relativeFolders:,=" "%") do (
        if not exist %targetEtc%\%%a mkdir %targetEtc%\%%a
    )

    FOR %%a in ("%relativeFilesNoOverwrite:,=" "%") do (
        CALL :copyFile %srcEtc%\%%a %targetEtc%\%%a no_overwrite
    )

    FOR %%a in ("%relativeFilesReplace:,=" "%") do (
        CALL :copyFile %srcEtc%\%%a %targetEtc%\%%a overwrite no_display
    )
    @endlocal
EXIT /B 0

:prepareTomcat
    if NOT exist %CATALINA_HOME%\bin\setenv.bat (
        CALL :log "Copying setenv.bat from %JF_PRODUCT_HOME%\app\misc to %CATALINA_HOME%\bin"
        copy /Y %JF_PRODUCT_HOME%\app\misc\service\setenv.bat %CATALINA_HOME%\bin\setenv.bat >nul
    )
EXIT /B 0

:configureServerXml
    set httpsEnabledKey=artifactory.tomcat.httpsConnector.enabled
    set httpsCrtYamlKey=artifactory.tomcat.httpsConnector.certificateFile
    set httpsPvtYamlKey=artifactory.tomcat.httpsConnector.certificateKeyFile
    if exist %CATALINA_HOME%\conf\server.xml (
        copy /Y %CATALINA_HOME%\conf\server.xml %CATALINA_HOME%\conf\server.xml.orig >nul
    )
    CALL %~dp0\systemYamlHelper.bat :getSystemValue !httpsEnabledKey!
    set https_enabled=!YAML_VALUE!
    IF [!https_enabled!] == [true] (
        copy /Y %JF_PRODUCT_HOME%\app\misc\tomcat\server.xml.https.template %CATALINA_HOME%\conf\server.xml >nul

        CALL %~dp0\systemYamlHelper.bat :getSystemValue !httpsCrtYamlKey! NOT_SET false
        set https_crt_value=!YAML_VALUE!
        CALL %~dp0\systemYamlHelper.bat :getSystemValue !httpsPvtYamlKey! NOT_SET false
        set https_pvtkey_value=!YAML_VALUE!
        set routerKeysPath=%JF_PRODUCT_HOME%\var\data\router\keys
        set rtSSLPath=%JF_PRODUCT_HOME%\var\etc\artifactory\security\ssl

        Rem check for private key and cert files
        CALL :checkCertsAndKeysInSystemYaml !rtSSLPath! !https_crt_value! !https_pvtkey_value! !httpsCrtYamlKey! !httpsPvtYamlKey! !routerKeysPath! || goto :EOF

    ) ELSE (
        copy /Y %JF_PRODUCT_HOME%\app\misc\tomcat\server.xml.template %CATALINA_HOME%\conf\server.xml >nul
    )
    CALL %~dp0\generateServerXml.bat || goto error
EXIT /B 0

Rem Attempting to use router's certificate and private key to artifactory
:useRoutersCertsAndKeys
    set rtSSLPath=%~1
    set routerKeysPath=%~2
    
    CALL :log "Attempting to use router's default server.crt and server.key"
    IF not exist !rtSSLPath! mkdir !rtSSLPath!
    IF not exist !routerKeysPath!\server.crt (
        CALL :warn "Router's !routerKeysPath!\server.crt is not generated to copy into pre-defined location !rtSSLPath!, please try restarting artifactory"
    ) ELSE (
        CALL :log "Copying !routerKeysPath!\server.crt to !rtSSLPath!\server.crt"
        copy /Y  !routerKeysPath!\server.crt !rtSSLPath!\server.crt  >nul 
    ) 
    IF not exist !routerKeysPath!\server.key (
        CALL :warn "Router's !routerKeysPath!\server.key is not generated to copy into pre-defined location !rtSSLPath!, please try restarting artifactory"
    ) ELSE (
        CALL :log "Copying !routerKeysPath!\server.key to !rtSSLPath!\server.key"
        copy /Y !routerKeysPath!\server.key !rtSSLPath!\server.key >nul
    )
EXIT /B 0

REM check certificate and private key in pre-defined location
:checkForCertsAndKeys
    set rtSSLPath=%~1
    set routerKeysPath=%~2

    if exist !rtSSLPath!\server.crt (
        if exist !rtSSLPath!\server.key (
            CALL :log "Using server.crt and server.key from pre-definied location !rtSSLPath!"
            goto :checkForCertsAndKeysEnd
        )  
    )
    if not exist !rtSSLPath!\server.crt (
        if exist !rtSSLPath!\server.key (
            set JF_ERROR_MSG="Private key found, but could not find certificate in path !rtSSLPath!"
            goto :errorExit
        )
    )
    if not exist !rtSSLPath!\server.key (
        if exist !rtSSLPath!\server.crt (
            set JF_ERROR_MSG="Certificate found, but could not find private key in path !rtSSLPath!"
            goto :errorExit
        )
    )
    if not exist !rtSSLPath!\server.crt (
        if not exist !rtSSLPath!\server.key (
            goto :useRoutersCertsAndKeys !rtSSLPath! !routerKeysPath!
        )
    )
    :checkForCertsAndKeysEnd
EXIT /B 0

Rem Check for private key and certificate configured in system.yaml 
:checkCertsAndKeysInSystemYaml
    set rtSSLPath=%~1
    set https_crt_value=%~2
    set https_pvtkey_value=%~3
    set httpsCrtYamlKey=%~4
    set httpsPvtYamlKey=%~5
    set routerKeysPath=%~6
     
    IF [!https_crt_value!] == [NOT_SET] (
        IF not [!https_pvtkey_value!] == [NOT_SET] (
            IF not exist !https_pvtkey_value! (
                set JF_ERROR_MSG="Could not find private key in path !https_pvtkey_value!. Update !httpsPvtYamlKey! with correct <private key path> in %JF_PRODUCT_HOME%\var\etc\system.yaml"
                goto :errorExit 
            )
            set JF_ERROR_MSG="Private key path is set, but certificate path is not set. Configure !httpsCrtYamlKey! with <certificate path> in %JF_PRODUCT_HOME%\var\etc\system.yaml"
            goto :errorExit
        )
    )
    IF [!https_pvtkey_value!] == [NOT_SET] (
        IF not [!https_crt_value!] == [NOT_SET] (
            IF not exist !https_crt_value! (
                set JF_ERROR_MSG="Could not find certificate in path !https_crt_value!. Update !httpsCrtYamlKey! with correct <certificate path> in %JF_PRODUCT_HOME%\var\etc\system.yaml"
                goto :errorExit 
            )
            set JF_ERROR_MSG="Certificate path is set, but private key path is not set. Configure !httpsPvtYamlKey! with <private key path> in %JF_PRODUCT_HOME%\var\etc\system.yaml"
            goto :errorExit
        )
    )
    IF not [!https_crt_value!/!https_pvtkey_value!] == [NOT_SET/NOT_SET] (
        Rem Private key and certificate configured in system.yaml
        CALL :log "Using private key and certificate files from the path configured in system.yaml"  
    )
    IF [!https_crt_value!/!https_pvtkey_value!] == [NOT_SET/NOT_SET] (
        Rem check certificate and private key in pre-defined location
        CALL :checkForCertsAndKeys !rtSSLPath! !routerKeysPath! || goto :EOF 
    )
EXIT /B 0
    
Rem get time with defined format
:timeFormat
    FOR /f "tokens=2 delims==" %%a IN ('wmic OS Get localdatetime /value') DO SET "dt=%%a"
	SET "YY=%dt:~2,2%" & SET "YYYY=%dt:~0,4%" & SET "MM=%dt:~4,2%" & SET "DD=%dt:~6,2%"
	SET "HH=%dt:~8,2%" & SET "Min=%dt:~10,2%" & SET "Sec=%dt:~12,2%" & SET "MSec=%dt:~9,2%"
	SET "datestamp=%YYYY%%MM%%DD%" & SET "timestamp=%HH%%Min%%Sec%"
	SET "fullstamp=%YYYY%-%MM%-%DD%T%HH%:%Min%:%Sec%:%MSec%Z"
EXIT /B 0

:: Common methods 
:log
    CALL :timeFormat || GOTO :EOF
    SET time=!fullstamp!
    echo !time! [INFO ] [!scriptName! ] [main]- %~1
EXIT /B 0

:warn
    CALL :timeFormat || GOTO :EOF
    SET time=!fullstamp!
    echo !time! [WARN ] [!scriptName! ] [main]- %~1
EXIT /B 0

:errorExit
    CALL :timeFormat || GOTO :EOF
    SET time=!fullstamp!
    echo.
    echo !time! [ERROR ] [!scriptName! ] [main]- %JF_ERROR_MSG%
    echo.
    @endlocal
exit /b 1

:error
    @endlocal
exit /b 1

EXIT /B 0