/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.webapp.servlet;

import com.jfrog.sysconf.SysConfigConstants;
import org.apache.commons.lang.StringUtils;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.opentracing.ArtifactoryJaegerTracerConfiguration;
import org.artifactory.opentracing.ArtifactoryTracer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

/**
 * @author yoavl
 */
public class ArtifactoryHomeConfigListener implements ServletContextListener {
    private static final Logger log = LoggerFactory.getLogger(ArtifactoryHomeConfigListener.class);

    @Override
    public void contextInitialized(ServletContextEvent event) {
        try {
            ServletContext servletContext = event.getServletContext();

            // Start a tracing span for the Artifactory startup
            ArtifactoryTracer artifactoryTracer = new ArtifactoryTracer(
                    ArtifactoryJaegerTracerConfiguration.initTracer());
            artifactoryTracer.startSpanOnCurrentThread("Artifactory Init");
            // Create ArtifactoryHome
            ArtifactoryHome artifactoryHome = initArtifactoryHome(servletContext);
            BasicConfigManagers basicConfigManagers = new BasicConfigManagers(artifactoryHome, artifactoryTracer);
            basicConfigManagers.initialize();

            basicConfigManagers.addServletAttributes(servletContext);
            basicConfigManagers.inheritInitParamsAsConstantValues(servletContext);
        } catch (RuntimeException e) {
            String reason = StringUtils.isNotBlank(e.getMessage()) ? e.getMessage() : "";
            log.error("Failed initializing Home. {}", reason);
            if (log.isDebugEnabled()) {
                log.debug("Failed initializing Home. ", e);
            }
            throw e;
        }
    }

    private ArtifactoryHome initArtifactoryHome(ServletContext servletContext) {
        String homeDir = getHomeDirForTesting(servletContext);
        ArtifactoryHome artifactoryHome = new ArtifactoryHome(homeDir);
        log.info("Resolved Home: '{}", artifactoryHome.getHomeDir().getAbsolutePath());
        ArtifactoryHome.bind(artifactoryHome);
        return artifactoryHome;
    }

    static String getHomeDirForTesting(ServletContext servletContext) {
        return servletContext.getInitParameter(SysConfigConstants.KEY_JF_PRODUCT_HOME);
    }

    @Override
    public void contextDestroyed(ServletContextEvent sce) {
        // nop
    }
}
